/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package GUI;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import util.*;

/*********************************************************
* Class for creating the configuration tool GUI
* The class creates three tabbed panes with different
* configuration options
*
* @author Olav Haugan
* @author Tarana Damania
* @author David McKinnon
* @author Kevin Dorow
*********************************************************/
public class MQCFrame extends JFrame
{
  private static final int NO_TAB = 6; //Number of "tabs"

  public static final String PRIMITIVE_TYPES_HDR = "Primary types";
  public static final String COMPLEX_TYPES_HDR = "Complex types";
  public static final String EXCEPTION_TYPES_HDR = "Exception types";
  
  public static final String MBPS = "Mbps";
  
  //Name of the tabs
  private static final String TAB1STR           = "Data types";
  private static final String TAB3STR           = "Comm. & Protocols";
  private static final String TAB4STR           = "Misc";
  private static final String TAB_SECURITY_STR  = "Security";
  private static final String TAB_FAULT_TOL_STR = "Fault Tolerance";
  private static final String TAB_OTHER_QOS_STR = "Other QoS";

  //Tha action listener associated with the menu and buttons 
  private Action actionL;
  private JTextField secCipherKeyLenText;     // needs to be here for scoping purposes
  private JLabel secCipherKeyHintsLabel;      // needs to be here for scoping purposes
  private ActionListener secCiphersActionL;   //FIXME: Is this correct??
  private ActionListener secMDActionL;        //FIXME: Is this correct??
  private JTextField secMACKeyLenText;        // needs to be here for scoping purposes
  private JLabel secMACKeyHintsLabel;         // needs to be here for scoping purposes
  private JLabel secHMACKeyHintsLabel;        // needs to be here for scoping purposes
  private ActionListener secMACActionL;       //FIXME: Is this correct??
  private JTextField secAuditLogLenText;      // needs? to be here?
  private ActionListener secAuditActionL;       //FIXME: Is this correct??
  
  //The properties where the configuration is saved
  private Properties MQCProperties;

  //The primitive data types check boxes
  private JCheckBox primTypesChkBox[];  
  //The complex data types check boxes
  private JCheckBox complexTypesChkBox[];  
  //The exceptions check boxes.
  private JCheckBox exceptionTypesChkBox[];  

  //Check boxes for QoS
  private JCheckBox secChkBox;
  private JCheckBox secMDChkBox;
  private JCheckBox secMACChkBox;
  private JCheckBox secAuditChkBox;
  private JCheckBox RTChkBox;
  private JCheckBox NetChkBox;

  //The group of Security radiobuttons
  private ButtonGroup  secCipherButtonGroup;
  private JRadioButton secCipherButtons[];
  private ButtonGroup  secCipherModeButtonGroup;
  private JRadioButton secCipherModeButtons[];
  private ButtonGroup  secCipherPaddingButtonGroup;
  private JRadioButton secCipherPaddingButtons[];
//private JTextField   secCipherKeyLenText; // defined previously/elsewhere
  private ButtonGroup  secMDButtonGroup;
  private JRadioButton secMDButtons[];
  private ButtonGroup  secMACButtonGroup;
  private JRadioButton secMACButtons[];
  private ButtonGroup  secAuditButtonGroup;
  private JRadioButton secAuditButtons[];
  
  //The group of realtime radiobuttons
  private ButtonGroup realtimeButtons;

  //An vector of Network QoS checkboxes
  private Vector netQoSButtons;

  //The group of GIOP radiobuttons for the client side
  private ButtonGroup clientGIOPButtons;
  //The group of Transport radiobuttons for the client side
  private ButtonGroup clientTRButtons;

  //The group of GIOP checkboxes for the server side
  private JCheckBox serverGIOPChkBox[];  
  //The group of Transport checkboxes for the server side
  private JCheckBox serverTRChkBox[];  


  //The guaranteed bandwidth textfield
  private JTextField bandwidth;  
  //The message payload textfield
  private JTextField messagePayload;  
  //The # of interfaces textfield
  private JTextField noInterfaces;
  //The # of methods textfield
  private JTextField noMethods;
  //The group of hw radiobuttons
  private ButtonGroup hwButtons;
  //The button to force endianess
  private JCheckBox endianChkBox;
  //The combobox to select endianess
  private JComboBox endianComboBox;
  //The group of marshaling radiobuttons
  private ButtonGroup marshalButtons;
  private JCheckBox compressNamesChkbox;
  //CheckBox to force CLDC support
  private JCheckBox cldcChkBox;
  //Debug support
  private JCheckBox debugChkBox;
  private JCheckBox debugMemoryChkBox;
  private JCheckBox debugTimingChkBox;
  private JTextField debugLevel;
  
  //Icon for the open menu item/button
  private ImageIcon openIcon;
  //Icon for the save menu item/button
  private ImageIcon saveIcon;
  //Icon for the program
  private ImageIcon MQCIcon;

  //Icon for the check profile button (profile not checked or wrong)
  private ImageIcon chkProfOffIcon;
  //Icon for the check profile button (profile checked and OK)
  private ImageIcon chkProfOnIcon;

  //Check profile menu button
  private JMenuItem chkProfile;
  
  //Check profile menu button
  private JButton chkProfileBtn;

  private MyItemListener itemListener;
  private MyDocumentListener docListener;
  
  // Fault Tolerance GUI declarations
  private javax.swing.JPanel jPFaultTol9;
  private javax.swing.JPanel jPFaultTol3;
  private javax.swing.JRadioButton jRBOrderNone;
  private javax.swing.JCheckBox chkFTSTCP;
  private javax.swing.JCheckBox chkFTSUnUDP;
  private javax.swing.JPanel jPFaultTol4;
  private javax.swing.JPanel jPFaultTol10;
  private javax.swing.JCheckBox chkGroupCom;
  private javax.swing.JRadioButton jRBUniform;
  private javax.swing.JLabel jLblGCTransport;
  private javax.swing.JPanel jPFaultTol1;
  private javax.swing.JPanel jPFaultTol8;
  private javax.swing.JRadioButton jRBOrderCausal;
  private javax.swing.JRadioButton jRBGroupTransTCP;
  private javax.swing.JLabel jLblOrdering;
  private javax.swing.JLabel jLblUniform;
  private javax.swing.JTextField jTFNumTemporal;
  private javax.swing.JLabel jLblTimeOut;
  private javax.swing.JRadioButton jRBOrderTotal;
  private javax.swing.JRadioButton jRBOrderFIFO;
  private javax.swing.JPanel jPFaultTol6;
  private javax.swing.JTextField jTFTimeOut;
  private javax.swing.JRadioButton jRBGroupTransUDP;
  private javax.swing.JPanel jPFaultTol11;
  private javax.swing.JRadioButton jRBNonUniform;
  private javax.swing.JLabel jLblTemporal;
  private javax.swing.JPanel jPFaultTol5;
  private javax.swing.JCheckBox chkValue;
  private javax.swing.JScrollPane jSPFaultTol;
  private javax.swing.JCheckBox chkFTSSNWUDP;
  private javax.swing.JCheckBox chkFTSGBNUDP;
  private javax.swing.JCheckBox chkTemporal;
  private javax.swing.JCheckBox chkSpatial;
  private javax.swing.JPanel jPFaultTol2;
  private javax.swing.JPanel jPFaultTol12;
  private javax.swing.JPanel jPFaultTol7;
  private javax.swing.JCheckBox chkFTSSerial;
  private javax.swing.JCheckBox chkFTSOneWire;

  /*********************************************************
  * Constructor.
  *
  *********************************************************/
  public MQCFrame()
  {
    this(null);
  }
  
  /*********************************************************
  * Constructor. Initializes variables.
  * 
  * @param String The title of the window
  *        
  *********************************************************/
  public MQCFrame( String title )
  {
    super( title );
  }

  /*********************************************************
  * Populates the controls with the saved data
  *
  * @param boolean true if we want to update the properties
  *                from the controls, false otherwise
  *
  *********************************************************/
  public Properties getProperties(boolean save)
  {
    if(save)
      updateProperties();
    return MQCProperties;
  }
  
  /*********************************************************
  * Called automatically by the underlying framework.
  * Initializes the frame and create its contents. 
  *********************************************************/
  protected void frameInit()
  {
    super.frameInit();
    MQCIcon = loadImageFromJar("images/mqc.gif");
    openIcon = loadImageFromJar("images/open.gif");
    saveIcon = loadImageFromJar("images/save.gif");
    chkProfOffIcon = loadImageFromJar("images/chkprofoff.gif");
    chkProfOnIcon = loadImageFromJar("images/chkprofon.gif");

    itemListener = new MyItemListener(this);
    docListener = new MyDocumentListener(this);
        
    setIconImage(MQCIcon.getImage());    
    
    MQCProperties = new Properties();
    actionL = new Action(this);

    secCipherKeyLenText    = new JTextField(Constants.SEC_CIPHER_KEY_DEFS[0], 4);
    secCipherKeyHintsLabel = new JLabel(Constants.SEC_CIPHER_KEY_HINTS[0]);
    secCiphersActionL      = new SecCiphersActionListener(secCipherKeyLenText, secCipherKeyHintsLabel);
    secHMACKeyHintsLabel   = new JLabel(Constants.SEC_HMAC_KEY_HINTS[0]);
    secMDActionL           = new SecMDActionListener(secHMACKeyHintsLabel);
    secMACKeyLenText       = new JTextField(Constants.SEC_MAC_KEY_DEFS[0], 4);
    secMACKeyHintsLabel    = new JLabel(Constants.SEC_MAC_KEY_HINTS[0]);
    secMACActionL          = new SecMACActionListener(secMACKeyLenText, secMACKeyHintsLabel);
    secAuditLogLenText     = new JTextField(Constants.SEC_AUDIT_LOG_DEFS[0], 4);
    secAuditActionL        = new SecAuditActionListener(secAuditLogLenText);
    
    JMenuBar menuBar = createMenu();
    setJMenuBar( menuBar );
    
    createApplicationContent();
  }
  
  /*********************************************************
  * Creates the menu for the frame
  *********************************************************/
  protected JMenuBar createMenu()
  {
    JMenuBar menuBar = new JMenuBar();
    
    JMenu file = new JMenu("File");
    file.setMnemonic(KeyEvent.VK_F);
    
    JMenuItem open = new JMenuItem( "Open...", openIcon );
    open.setHorizontalTextPosition( SwingConstants.RIGHT );
    open.setMnemonic( KeyEvent.VK_O );
    open.setAccelerator( KeyStroke.getKeyStroke(KeyEvent.VK_O, Event.CTRL_MASK) );
    open.addActionListener( actionL );

    JMenuItem save = new JMenuItem( "Save", saveIcon );
    save.setHorizontalTextPosition( SwingConstants.RIGHT );
    save.setMnemonic( KeyEvent.VK_S );
    save.setAccelerator( KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.CTRL_MASK) );
    save.addActionListener( actionL );

    JMenuItem saveAs = new JMenuItem( "Save as...", null );
    saveAs.setHorizontalTextPosition( SwingConstants.RIGHT );
    saveAs.addActionListener( actionL );

    chkProfile = new JMenuItem( "Check profile", chkProfOffIcon );
    chkProfile.setHorizontalTextPosition( SwingConstants.RIGHT );
    chkProfile.addActionListener( actionL );

    JMenuItem exit = new JMenuItem( "Exit", null );
    exit.setHorizontalTextPosition( SwingConstants.RIGHT );
    exit.setMnemonic( KeyEvent.VK_X );
    exit.setAccelerator( KeyStroke.getKeyStroke(KeyEvent.VK_X, Event.CTRL_MASK) );
    exit.addActionListener( actionL );
    
    file.add( open );
    file.addSeparator();
    file.add( save );
    file.add( saveAs );
    file.addSeparator();
    file.add( chkProfile );
    file.addSeparator();
    file.add( exit );

    JMenu help = new JMenu("Help");
    help.setMnemonic(KeyEvent.VK_H);

    JMenuItem about = new JMenuItem( "About", null );
    about.setHorizontalTextPosition( SwingConstants.RIGHT );
    about.addActionListener( actionL );
    
    help.add(about);
    
    menuBar.add( file );
    menuBar.add( help );
    return menuBar;
  }
  
  /*********************************************************
  * Creates the tabbed panes and their contents
  *********************************************************/
  protected void createApplicationContent()
  {
    primTypesChkBox         = new JCheckBox[Constants.NO_PRIMITIVE_TYPES];  
    complexTypesChkBox      = new JCheckBox[Constants.NO_COMPLEX_TYPES];  
    exceptionTypesChkBox    = new JCheckBox[Constants.NO_EXCEPTION_TYPES];
    secCipherButtons        = new JRadioButton[Constants.SEC_CIPHERS.length];
    secCipherModeButtons    = new JRadioButton[Constants.SEC_CIPHER_MODES.length];
    secCipherPaddingButtons = new JRadioButton[Constants.SEC_CIPHER_PADDINGS.length];
    secMDButtons            = new JRadioButton[Constants.SEC_MD_ALGS.length];
    secMACButtons           = new JRadioButton[Constants.SEC_MAC_ALGS.length];
    secAuditButtons         = new JRadioButton[Constants.SEC_AUDIT_ALGS.length];
    
    serverTRChkBox = new JCheckBox[Constants.NO_TRANSPORTS];  
    serverGIOPChkBox = new JCheckBox[Constants.NO_PROTOCOLS];  
    
    //Create the tabbed panes
    JTabbedPane tabbedPane = createTabbedPane();
    Container content = getContentPane();
    content.add( BorderLayout.CENTER, tabbedPane );
    
    //Add open button to the frame
    final JPanel buttons = new JPanel();
    final JButton openBtn = new JButton("Open", openIcon);
    openBtn.addActionListener( actionL );
    buttons.add( openBtn );

    //Add save button to the frame
    final JButton saveBtn = new JButton("Save", saveIcon);
    saveBtn.addActionListener( actionL );
    buttons.add( saveBtn );

    //Add check profile button to the frame
    chkProfileBtn = new JButton("Check profile", chkProfOffIcon);
    chkProfileBtn.addActionListener( actionL );
    buttons.add( chkProfileBtn );

    //Add exit button to the frame
    final JButton exitBtn = new JButton("Exit");
    exitBtn.addActionListener( actionL );
    buttons.add( exitBtn );
    
    content.add( BorderLayout.SOUTH, buttons );
  }

  /*********************************************************
  * Creates the panels with their contents
  *********************************************************/
  private JTabbedPane createTabbedPane()
  {
    JTabbedPane tabbedPane = new JTabbedPane();

    tabbedPane.add(createPanel1(), TAB1STR);
    tabbedPane.add(createPanel3(), TAB3STR);
    tabbedPane.add(createPanel4(), TAB4STR);
    tabbedPane.add(createPanelSecurity(), TAB_SECURITY_STR);
    tabbedPane.add(createPanelFaultTol(), TAB_FAULT_TOL_STR);
    tabbedPane.add(createPanelOtherQoS(), TAB_OTHER_QOS_STR);
    return tabbedPane;
    
  }
  
  /*********************************************************
  * Creates panel 1
  * Contains check boxes for primitive types, complex
  * types, and exceptions
  *********************************************************/
  private JPanel createPanel1()
  {
    JPanel panel = new JPanel();
    panel.setLayout( new GridLayout(3,1) );
    
    JPanel panel1 = new JPanel();
    JPanel panel2 = new JPanel();
    JPanel panel3 = new JPanel();
    panel1.setLayout( new BorderLayout() );
    panel1.setBorder(BorderFactory.createLoweredBevelBorder());
    panel2.setLayout( new BorderLayout() );
    panel2.setBorder(BorderFactory.createLoweredBevelBorder());
    panel3.setLayout( new BorderLayout() );
    panel3.setBorder(BorderFactory.createLoweredBevelBorder());
    
    //Make a panel with primitive data type checkboxes
    JPanel primPanel = new JPanel();        
    primPanel.setLayout( new GridLayout(4,4) );
    for(int i = 0; i < Constants.NO_PRIMITIVE_TYPES; i++)
    {
      primTypesChkBox[i] = new JCheckBox(Constants.PRIMITIVE_TYPE[i], false);
      primTypesChkBox[i].addItemListener(itemListener);
      primPanel.add(primTypesChkBox[i]);
    }
    // Turn on default primative types
    primTypesChkBox[6].setSelected(true);
    JLabel label = new JLabel(PRIMITIVE_TYPES_HDR, SwingConstants.CENTER);
    label.setOpaque(true);
    panel1.add(label, BorderLayout.NORTH );
    panel1.add(primPanel, BorderLayout.CENTER );


    //Make a panel with complex data type checkboxes
    JPanel complexPanel = new JPanel();
    complexPanel.setLayout( new GridLayout(2,4) );
    for(int i = 0; i < Constants.NO_COMPLEX_TYPES; i++)
    {
      complexTypesChkBox[i] = new JCheckBox(Constants.COMPLEX_TYPE[i], false);
      complexTypesChkBox[i].addItemListener(itemListener);
      complexTypesChkBox[i].setEnabled(false);
      complexPanel.add(complexTypesChkBox[i]);
    }
    label = new JLabel(COMPLEX_TYPES_HDR, SwingConstants.CENTER);
    label.setOpaque(true);
    panel2.add(label, BorderLayout.NORTH );
    panel2.add(complexPanel, BorderLayout.CENTER );

    //Make a panel with exception type checkboxes
    JPanel exceptionsPanel = new JPanel();
    exceptionsPanel.setLayout( new GridLayout(0,2) );
    for(int i = 0; i < Constants.NO_EXCEPTION_TYPES; i++)
    {
      exceptionTypesChkBox[i] = new JCheckBox(Constants.EXCEPTION_TYPE[i], false);
      exceptionTypesChkBox[i].addItemListener(itemListener);
      exceptionsPanel.add(exceptionTypesChkBox[i]);
    }
    label = new JLabel(EXCEPTION_TYPES_HDR, SwingConstants.CENTER);
    label.setOpaque(true);
    panel3.add(label, BorderLayout.NORTH );
    panel3.add(exceptionsPanel, BorderLayout.CENTER );
    
    //Add the three panels to the main panel
    panel.add(panel1);
    panel.add(panel2);
    panel.add(panel3, BorderLayout.CENTER);
    return panel;
  }

  /*********************************************************
  * Creates panel Security
  * Contains buttons for fault tolerance, real time,
  * security, and network QoS
  *********************************************************/
  private JPanel createPanelSecurity()
  {
    JPanel panel = new JPanel();
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1.0;
    c.gridwidth = GridBagConstraints.REMAINDER;
    panel.setLayout(gridbag);

    // Security
    //----------
    // Security Label
    secChkBox = new JCheckBox(Constants.SECURITY, false);  
    secChkBox.addActionListener(new CheckBoxAction(this, secCipherButtonGroup));
    c.insets = new Insets(0, 0, 0, 0);
    gridbag.setConstraints(secChkBox, c);
    panel.add(secChkBox);
    
    //-- Security Cipher Label --
    JPanel secCipherPanel = new JPanel();
    secCipherPanel.setLayout(new GridLayout(3,5));
    JLabel secCipherLabel = new JLabel(Constants.SEC_CIPHER + ": ");
    secCipherPanel.add(secCipherLabel);
    //-- Security Cipher RadioButtons
    secCipherButtonGroup = new ButtonGroup();
    for (int i = 0;  i < Constants.SEC_CIPHERS.length;  i++) {
      secCipherButtons[i] = new JRadioButton(Constants.SEC_CIPHERS[i], false);
      secCipherButtons[i].setEnabled(true);
      secCipherButtons[i].addActionListener(secCiphersActionL);
      secCipherButtonGroup.add(secCipherButtons[i]);
      secCipherPanel.add(secCipherButtons[i]);
    }
    secCipherButtons[0].setSelected(true);
    c.insets = new Insets(0, 25, 0, 0);
    gridbag.setConstraints(secCipherPanel, c);
    panel.add(secCipherPanel);
    
    //-- Security Cipher Mode Label --
    JPanel secCipherModePanel = new JPanel();
    secCipherModePanel.setLayout(new GridLayout(1,5));
    JLabel secCipherModeLabel = new JLabel(Constants.SEC_CIPHER_MODE + ": ");
    secCipherModePanel.add(secCipherModeLabel);
    //-- Security Cipher Mode RadioButtons
    secCipherModeButtonGroup = new ButtonGroup();
    for (int i = 0;  i < Constants.SEC_CIPHER_MODES.length;  i++) {
      secCipherModeButtons[i] = new JRadioButton(Constants.SEC_CIPHER_MODES[i], false);
      secCipherModeButtons[i].setEnabled(true);
      secCipherModeButtons[i].addItemListener(itemListener);
      secCipherModeButtonGroup.add(secCipherModeButtons[i]);
      secCipherModePanel.add(secCipherModeButtons[i]);
    }
    secCipherModeButtons[0].setSelected(true);
//    c.insets = new Insets(0, 25, 0, 0);
//    gridbag.setConstraints(secCipherModePanel, c);
//    panel.add(secCipherModePanel);
    
    //-- Security Cipher Padding Label --
//    JPanel secCipherPaddingPanel = new JPanel();
//    secCipherPaddingPanel.setLayout(new GridLayout(1,5));
    JLabel secCipherPaddingLabel = new JLabel(Constants.SEC_CIPHER_PADDING + ": ");
//    secCipherPaddingPanel.add(secCipherPaddingLabel);
    secCipherModePanel.add(secCipherPaddingLabel);
    //-- Security Cipher Padding RadioButtons
    secCipherPaddingButtonGroup = new ButtonGroup();
    for (int i = 0;  i < Constants.SEC_CIPHER_PADDINGS.length;  i++) {
      secCipherPaddingButtons[i] = new JRadioButton(Constants.SEC_CIPHER_PADDINGS[i], false);
      secCipherPaddingButtons[i].setEnabled(true);
      secCipherPaddingButtons[i].addItemListener(itemListener);
      secCipherPaddingButtonGroup.add(secCipherPaddingButtons[i]);
//      secCipherPaddingPanel.add(secCipherPaddingButtons[i]);
      secCipherModePanel.add(secCipherPaddingButtons[i]);
    }
    // Set the second button (PKCS5) to true
    secCipherPaddingButtons[1].setSelected(true);
    c.insets = new Insets(0, 25, 0, 0);
//    gridbag.setConstraints(secCipherPaddingPanel, c);
//    panel.add(secCipherPaddingPanel);
    secCipherModePanel.add( new JLabel(" "));   // for good looks?
    gridbag.setConstraints(secCipherModePanel, c);
    panel.add(secCipherModePanel);
    
    //-- Security Cipher Key Length Label --
    JPanel secCipherKeyLenPanel = new JPanel();
    secCipherKeyLenPanel.setLayout(new GridLayout(1,5));
    JLabel secCipherKeyLenLabel = new JLabel(Constants.SEC_CIPHER_KEY_LEN + ": ");
    secCipherKeyLenPanel.add(secCipherKeyLenLabel);
    //-- Security Cipher Key Length TextField
    secCipherKeyLenText.getDocument().addDocumentListener(docListener);
    gridbag.setConstraints(secCipherKeyLenText, c);
    secCipherKeyLenText.setEditable(true);
    secCipherKeyLenPanel.add(secCipherKeyLenText);
    //-- Security Cipher Key Hints Label
    secCipherKeyLenPanel.add(secCipherKeyHintsLabel);

    c.insets = new Insets(0, 25, 0, 0);
    gridbag.setConstraints(secCipherKeyLenPanel, c);
    panel.add(secCipherKeyLenPanel);


    //-- Security Message Digests
    secMDChkBox = new JCheckBox(Constants.SECURITY_MD, false);  
    secMDChkBox.addActionListener(new CheckBoxAction(this, secMDButtonGroup));
    c.insets = new Insets(5, 0, 0, 0);
    gridbag.setConstraints(secMDChkBox, c);
    panel.add(secMDChkBox);

    //-- Security MD Algs --
    JPanel secMDPanel = new JPanel();
    secMDPanel.setLayout(new GridLayout(3,5));
    JLabel secMDLabel = new JLabel(Constants.SEC_MD_LABEL + ": ");
    secMDPanel.add(secMDLabel);
    //-- Security Message Digest RadioButtons
    secMDButtonGroup = new ButtonGroup();
    for (int i = 0;  i < Constants.SEC_MD_ALGS.length;  i++) {
      secMDButtons[i] = new JRadioButton(Constants.SEC_MD_ALGS[i], false);
      secMDButtons[i].setEnabled(true);
      secMDButtons[i].addActionListener(secMDActionL);
      secMDButtonGroup.add(secMDButtons[i]);
      secMDPanel.add(secMDButtons[i]);
    }
    secMDButtons[0].setSelected(true);
    c.insets = new Insets(0, 25, 0, 0);
    gridbag.setConstraints(secMDPanel, c);
    panel.add(secMDPanel);


    //-- Security MAC
    secMACChkBox = new JCheckBox(Constants.SECURITY_MAC, false);  
    secMACChkBox.addActionListener(new CheckBoxAction(this, secMACButtonGroup));
    c.insets = new Insets(5, 0, 0, 0);
    gridbag.setConstraints(secMACChkBox, c);
    panel.add(secMACChkBox);
    
    //-- Security MAC Algs --
    JPanel secMACPanel = new JPanel();
    secMACPanel.setLayout(new GridLayout(1,5));
    JLabel secMACLabel = new JLabel(Constants.SEC_MAC_LABEL + ": ");
    secMACPanel.add(secMACLabel);
    //-- Security MAC RadioButtons
    secMACButtonGroup = new ButtonGroup();
    for (int i = 0;  i < Constants.SEC_MAC_ALGS.length;  i++) {
      secMACButtons[i] = new JRadioButton(Constants.SEC_MAC_ALGS[i], false);
      secMACButtons[i].setEnabled(true);
      secMACButtons[i].addActionListener(secMACActionL);
      secMACButtonGroup.add(secMACButtons[i]);
      secMACPanel.add(secMACButtons[i]);
    }
    secMACButtons[0].setSelected(true);
    c.insets = new Insets(0, 25, 0, 0);
    gridbag.setConstraints(secMACPanel, c);
    panel.add(secMACPanel);

    //-- Security MAC Key Length Label --
    JPanel secMACKeyLenPanel = new JPanel();
    secMACKeyLenPanel.setLayout(new GridLayout(1,5));
    JLabel secMACKeyLenLabel = new JLabel(Constants.SEC_MAC_KEY_LEN + ": ");
    secMACKeyLenPanel.add(secMACKeyLenLabel);
    //-- Security MAC Key Length TextField
    secMACKeyLenText.getDocument().addDocumentListener(docListener);
    gridbag.setConstraints(secMACKeyLenText, c);
    secMACKeyLenText.setEditable(true);
    secMACKeyLenPanel.add(secMACKeyLenText);
    //-- Security MAC & HMAC Key Hints Label
    secMACKeyLenPanel.add(secMACKeyHintsLabel);
    secMACKeyLenPanel.add(secHMACKeyHintsLabel);

    c.insets = new Insets(0, 25, 0, 0);
    gridbag.setConstraints(secMACKeyLenPanel, c);
    panel.add(secMACKeyLenPanel);

    //-- Security Audit
    secAuditChkBox = new JCheckBox(Constants.SECURITY_AUDIT, false);  
    c.insets = new Insets(5, 0, 0, 0);
    gridbag.setConstraints(secAuditChkBox, c);
    panel.add(secAuditChkBox);

    //-- Security Audit Algs --
    JPanel secAuditPanel = new JPanel();
    secAuditPanel.setLayout(new GridLayout(1,5));
    JLabel secAuditLabel = new JLabel(Constants.SEC_AUDIT_LABEL + ": ");
    secAuditPanel.add(secAuditLabel);
    //-- Security Audit RadioButtons
    secAuditButtonGroup = new ButtonGroup();
    for (int i = 0;  i < Constants.SEC_AUDIT_ALGS.length;  i++) {
      secAuditButtons[i] = new JRadioButton(Constants.SEC_AUDIT_ALGS[i], false);
      secAuditButtons[i].setEnabled(true);
      secAuditButtons[i].addActionListener(secAuditActionL);
      secAuditButtonGroup.add(secAuditButtons[i]);
      secAuditPanel.add(secAuditButtons[i]);
    }
    secAuditButtons[0].setSelected(true);

    //-- Audit Log Length Label --
    JLabel secAuditLogLenLabel = new JLabel(Constants.SEC_AUDIT_LOG_LEN + ": ");
    secAuditPanel.add(secAuditLogLenLabel);
    //-- Audit Log Length TextField
    secAuditLogLenText.getDocument().addDocumentListener(docListener);
    gridbag.setConstraints(secAuditLogLenText, c);
    secAuditLogLenText.setEditable(true);
    secAuditPanel.add(secAuditLogLenText);

    c.insets = new Insets(0, 25, 0, 0);
    gridbag.setConstraints(secAuditPanel, c);
    panel.add(secAuditPanel);


    return panel;
  }

  //-- Update the Key Length Hints when the ciphers change  
  class SecCiphersActionListener implements java.awt.event.ActionListener {
    private JTextField keyLenText;
    private JLabel     hintLabel;
    SecCiphersActionListener (JTextField text, JLabel label) {
      keyLenText = text;
      hintLabel  = label;
    }
    public void actionPerformed (java.awt.event.ActionEvent evt) {
      for (int i = 0;  i < Constants.SEC_CIPHERS.length;  i++) {
        if (secCipherButtons[i].isSelected()) {
          keyLenText.setText(Constants.SEC_CIPHER_KEY_DEFS[i]);
          hintLabel.setText(Constants.SEC_CIPHER_KEY_HINTS[i]);
          return;
        }
      }
    }
  }

  //-- Update the HMAC Block Length Hints when the Message Digests change  
  class SecMDActionListener implements java.awt.event.ActionListener {
    private JLabel hintLabel;
    SecMDActionListener (JLabel label) {
      hintLabel  = label;
    }
    public void actionPerformed (java.awt.event.ActionEvent evt) {
      for (int i = 0;  i < Constants.SEC_MD_ALGS.length;  i++) {
        if (secMDButtons[i].isSelected()) {
          hintLabel.setText(Constants.SEC_HMAC_KEY_HINTS[i]);
          return;
        }
      }
    }
  }
  
  //-- Update the Key Length Hints when the MACs change  
  class SecMACActionListener implements java.awt.event.ActionListener {
    private JTextField keyLenText;
    private JLabel     hintLabel;
    SecMACActionListener (JTextField text, JLabel label) {
      keyLenText = text;
      hintLabel  = label;
    }
    public void actionPerformed (java.awt.event.ActionEvent evt) {
      for (int i = 0;  i < Constants.SEC_MAC_ALGS.length;  i++) {
        if (secMACButtons[i].isSelected()) {
          keyLenText.setText(Constants.SEC_MAC_KEY_DEFS[i]);
          hintLabel.setText(Constants.SEC_MAC_KEY_HINTS[i]);
          return;
        }
      }
    }
  }

  //-- Update the Audit Log Length when the Audit algs change  
  class SecAuditActionListener implements java.awt.event.ActionListener {
    private JTextField keyLenText;
    SecAuditActionListener (JTextField text) {
      keyLenText = text;
    }
    public void actionPerformed (java.awt.event.ActionEvent evt) {
      for (int i = 0;  i < Constants.SEC_AUDIT_ALGS.length;  i++) {
        if (secAuditButtons[i].isSelected()) {
          keyLenText.setText(Constants.SEC_AUDIT_LOG_DEFS[i]);
          return;
        }
      }
    }
  }

  
  /*********************************************************
  * Creates panel Other QoS
  * Contains buttons for real time and network QoS
  *********************************************************/
  private JPanel createPanelOtherQoS()
  {
    JPanel panel = new JPanel();
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1.0;
    c.gridwidth = GridBagConstraints.REMAINDER;
    panel.setLayout(gridbag);

    //-- RealTime
    //-------------
    realtimeButtons = new ButtonGroup();
    JRadioButton RTsoft = new JRadioButton(Constants.RT_SOFT, true);
    RTsoft.setEnabled(false);
    RTsoft.addItemListener(itemListener);
    realtimeButtons.add(RTsoft);
    JRadioButton RThard = new JRadioButton(Constants.RT_HARD, false);
    RThard.setEnabled(false);
    RThard.addItemListener(itemListener);
    realtimeButtons.add(RThard);

    RTChkBox = new JCheckBox(Constants.REALTIME, false);  
    RTChkBox.addActionListener(new CheckBoxAction(this, realtimeButtons));

    JPanel rtPanel = new JPanel();
    rtPanel.setLayout(new GridLayout(1,4));

    rtPanel.add(RTChkBox);
    rtPanel.add(RTsoft);
    rtPanel.add(RThard);
    JLabel rtspace = new JLabel(" ");
    c.weightx = 1;
    gridbag.setConstraints(rtspace, c);
    rtPanel.add(rtspace);

    c.insets  = new Insets(10, 0, 0, 0);
    c.weightx = 1;
    gridbag.setConstraints(rtPanel, c);
    panel.add(rtPanel);
    //Realtime end

    //-- Network QoS
    //----------------
    netQoSButtons = new Vector(2,2);
    JCheckBox BWChkBox = new JCheckBox(Constants.NET_GUARANTEED_BW, false);
    BWChkBox.setEnabled(false);
    netQoSButtons.add(BWChkBox);
    JCheckBox LLChkBox = new JCheckBox(Constants.NET_LOW_LATENCY, false);
    LLChkBox.setEnabled(false);
    LLChkBox.addItemListener(itemListener);
    netQoSButtons.add(LLChkBox);
    
    c.insets = new Insets(0, 0, 0, 0);
    c.gridwidth = 1;
    c.weightx = 0;

    //Add button, textfield, and text to a panel
    JPanel p = new JPanel(gridbag);
    gridbag.setConstraints(BWChkBox, c);
    p.add(BWChkBox);

    bandwidth = new JTextField("0", 4);
    bandwidth.getDocument().addDocumentListener(docListener);
    gridbag.setConstraints(bandwidth, c);
    bandwidth.setEditable(false);
    p.add(bandwidth);

    BWChkBox.addActionListener( new CheckBoxAction(this, bandwidth));

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.weightx = 1;
    JLabel MBPSLabel = new JLabel(MBPS);
    gridbag.setConstraints(MBPSLabel, c);
    p.add(MBPSLabel);

    NetChkBox = new JCheckBox(Constants.NETWORK, false);  
    NetChkBox.addActionListener(new CheckBoxAction(this, netQoSButtons));

    c.insets = new Insets(0, 0, 0, 0);
    gridbag.setConstraints(NetChkBox, c);
    panel.add(NetChkBox);

    c.insets = new Insets(0, 20, 0, 0);
    gridbag.setConstraints(p, c);
    panel.add(p);

    gridbag.setConstraints(LLChkBox, c);
    panel.add(LLChkBox);
    //Network QoS end

    return panel;
  }
  

  /*********************************************************
  * Creates panel 3
  * Contains buttons for transport and comm. protocols
  *********************************************************/
  private JPanel createPanel3()
  {
    JPanel panel = new JPanel();
    
    JPanel panel1 = new JPanel();
    panel1.setBorder(BorderFactory.createLoweredBevelBorder());
    JPanel panel2 = new JPanel();
    panel2.setBorder(BorderFactory.createLoweredBevelBorder());

    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1.0;
    c.gridwidth = GridBagConstraints.REMAINDER;
    panel1.setLayout(gridbag);
    panel2.setLayout(gridbag);
    
    //Panel 1, client side
    //Transport system
    clientTRButtons = new ButtonGroup();
    JRadioButton TRTCP = new JRadioButton(Constants.TR_TCP, true);
    TRTCP.addItemListener(itemListener);
    clientTRButtons.add(TRTCP);

    JRadioButton TRUNRELIABLEUDP = new JRadioButton(Constants.TR_UNRELIABLE_UDP, true);
    TRUNRELIABLEUDP.addItemListener(itemListener);
    clientTRButtons.add(TRUNRELIABLEUDP);

    JRadioButton TRGOBACKNUDP = new JRadioButton(Constants.TR_GOBACKN_UDP, true);
    TRGOBACKNUDP.addItemListener(itemListener);
    clientTRButtons.add(TRGOBACKNUDP);

    JRadioButton TRSTOPANDWAITUDP = new JRadioButton(Constants.TR_STOPANDWAIT_UDP, true);
    TRSTOPANDWAITUDP.addItemListener(itemListener);
    clientTRButtons.add(TRSTOPANDWAITUDP);
    
    JRadioButton TRSerial = new JRadioButton(Constants.TR_SERIAL, true);
    TRSerial.addItemListener(itemListener);
    clientTRButtons.add(TRSerial);
    JRadioButton TR1Wire = new JRadioButton(Constants.TR_1WIRE, true);
    TR1Wire.addItemListener(itemListener);
    clientTRButtons.add(TR1Wire);

    JLabel TRLabel = new JLabel(Constants.TR);  
    c.insets = new Insets(5, 0, 0, 0);
    gridbag.setConstraints(TRLabel, c);
    panel1.add(TRLabel);

    c.insets = new Insets(0, 20, 0, 0);

    gridbag.setConstraints(TRTCP, c);
    panel1.add(TRTCP);

    gridbag.setConstraints(TRUNRELIABLEUDP, c);
    panel1.add(TRUNRELIABLEUDP);
    
    gridbag.setConstraints(TRGOBACKNUDP, c);
    panel1.add(TRGOBACKNUDP);
    
    gridbag.setConstraints(TRSTOPANDWAITUDP, c);
    panel1.add(TRSTOPANDWAITUDP);

    gridbag.setConstraints(TRSerial, c);
    panel1.add(TRSerial);

    gridbag.setConstraints(TR1Wire, c);
    panel1.add(TR1Wire);
    //Transport system end    

    //GIOP
    clientGIOPButtons = new ButtonGroup();
    JRadioButton GIOPGIOP = new JRadioButton(Constants.GIOP_GIOP, true);
    GIOPGIOP.addItemListener(itemListener);
    clientGIOPButtons.add(GIOPGIOP);
    JRadioButton GIOPLite = new JRadioButton(Constants.GIOP_LITE, true);
    GIOPLite.addItemListener(itemListener);
    clientGIOPButtons.add(GIOPLite);
    JRadioButton GIOPMQC = new JRadioButton(Constants.GIOP_MQC, true);
    GIOPMQC.addItemListener(itemListener);
    clientGIOPButtons.add(GIOPMQC);

    JLabel GIOPLabel = new JLabel(Constants.GIOP);  
    c.insets = new Insets(5, 0, 0, 0);
    gridbag.setConstraints(GIOPLabel, c);
    panel1.add(GIOPLabel);

    c.insets = new Insets(0, 20, 0, 0);
    gridbag.setConstraints(GIOPGIOP, c);
    panel1.add(GIOPGIOP);
    gridbag.setConstraints(GIOPLite, c);
    panel1.add(GIOPLite);
    gridbag.setConstraints(GIOPMQC, c);
    panel1.add(GIOPMQC);
    //GIOP end

    //Panel 2, server side
    //Transport system
    c.insets = new Insets(5, 0, 0, 0);
    TRLabel = new JLabel(Constants.TR);  
    gridbag.setConstraints(TRLabel, c);
    panel2.add(TRLabel);
    c.insets = new Insets(0, 20, 0, 0);
    
    for(int i = 0; i < Constants.NO_TRANSPORTS; i++)
    {
      serverTRChkBox[i] = new JCheckBox(Constants.TRANSPORT_TYPE[i], false);
      serverTRChkBox[i].addItemListener(itemListener);
      gridbag.setConstraints(serverTRChkBox[i], c);
      panel2.add(serverTRChkBox[i]);
    }
    // set the default(s)
    serverTRChkBox[0].setSelected(true);     // TCP/IP
    //Transport system end    

    //GIOP
    c.insets = new Insets(5, 0, 0, 0);
    GIOPLabel = new JLabel(Constants.GIOP);  
    gridbag.setConstraints(GIOPLabel, c);
    panel2.add(GIOPLabel);
    c.insets = new Insets(0, 20, 0, 0);

    for(int i = 0; i < Constants.NO_PROTOCOLS; i++)
    {
      serverGIOPChkBox[i] = new JCheckBox(Constants.PROTOCOL_TYPE[i], false);
      serverGIOPChkBox[i].addItemListener(itemListener);
      gridbag.setConstraints(serverGIOPChkBox[i], c);
      panel2.add(serverGIOPChkBox[i]);
    }
    // set the default(s)
    serverGIOPChkBox[0].setSelected(true);     // GIOP
    //GIOP end

    GridBagLayout gridbagMain = new GridBagLayout();
    GridBagConstraints cMain = new GridBagConstraints();
    cMain.fill = GridBagConstraints.BOTH;
    cMain.weightx = 1.0;
    cMain.gridwidth = GridBagConstraints.RELATIVE;
    panel.setLayout(gridbagMain);


    JLabel label = new JLabel(Constants.CLIENT_HDR, SwingConstants.CENTER);
    label.setOpaque(true);
    gridbagMain.setConstraints(label, cMain);
    panel.add(label);
    cMain.gridwidth = GridBagConstraints.REMAINDER;
    label = new JLabel(Constants.SERVER_HDR, SwingConstants.CENTER);
    label.setOpaque(true);
    gridbagMain.setConstraints(label, cMain);
    panel.add(label);

    cMain.gridwidth = GridBagConstraints.RELATIVE;
    gridbagMain.setConstraints(panel1, cMain);
    panel.add(panel1);
    cMain.gridwidth = GridBagConstraints.REMAINDER;
    gridbagMain.setConstraints(panel2, cMain);
    panel.add(panel2);
    return panel;
  }
  
  /*********************************************************
  * Creates panel 4
  * Contains options for message payload, number of 
  * interfaces and methods, compressing method names,
  * and unmarshal/marshal options.
  * Added support for CLDC devices and Debuging
  *********************************************************/
  private JPanel createPanel4()
  {
    JPanel panel = new JPanel();
    panel.setLayout( new GridLayout(4,1) );
    
    JPanel panel1 = new JPanel();
    JPanel panel2 = new JPanel();
//    JPanel panel3 = new JPanel();
    JPanel panel4 = new JPanel();
//    JPanel panel5 = new JPanel();
    JPanel panel6 = new JPanel();
    
    panel1.setLayout( new GridLayout(3,3) );
    panel1.setBorder(BorderFactory.createLoweredBevelBorder());
    panel2.setLayout( new GridLayout(3,1) );
    panel2.setBorder(BorderFactory.createLoweredBevelBorder());
//    panel3.setLayout( new GridLayout(1,1) );
//    panel3.setBorder(BorderFactory.createLoweredBevelBorder());
    panel4.setLayout( new GridLayout(3,1) );
    panel4.setBorder(BorderFactory.createLoweredBevelBorder());
//    panel5.setLayout( new GridLayout(1,3) );
//    panel5.setBorder(BorderFactory.createLoweredBevelBorder());
    panel6.setLayout( new GridLayout(2,1) );
    panel6.setBorder(BorderFactory.createLoweredBevelBorder());
        
    //Begin message payload, # of interfaces and methods
    panel1.add(new JLabel(Constants.MSG_PAYLOAD));
    messagePayload = new JTextField("0");
    messagePayload.getDocument().addDocumentListener(docListener);
    panel1.add(messagePayload);
    panel1.add(new JLabel("bytes  (0==Variable length)"));
    panel1.add(new JLabel(Constants.NO_INTERFACES));
    
    noInterfaces = new JTextField("0");
    noInterfaces.getDocument().addDocumentListener(docListener);
    noInterfaces.setEditable(true);
    panel1.add(noInterfaces);
    
    panel1.add(new JLabel()); //dummy label to make things look good

    panel1.add(new JLabel(Constants.NO_METHODS));
    noMethods = new JTextField("0");
    noMethods.getDocument().addDocumentListener(docListener);
    noMethods.setEditable(true);
    panel1.add(noMethods);
    //End message payload, # of interfaces and methods
    
    //Begin homogenous/heterogeneous HW
    hwButtons = new ButtonGroup();
    JRadioButton homogeneous = new JRadioButton(Constants.HOMOGENEOUS_HW, true);
    homogeneous.addItemListener(itemListener);
    hwButtons.add(homogeneous);
    JRadioButton heterogeneous = new JRadioButton(Constants.HETEROGENEOUS_HW, false);
    heterogeneous.addItemListener(itemListener);
    hwButtons.add(heterogeneous);
    panel2.add(homogeneous);
    panel2.add(heterogeneous);
    JPanel endianPanel = new JPanel();
    
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.BOTH;
    c.gridwidth = 1;
    c.weightx = 0;

    endianPanel.setLayout(gridbag);
    
    endianComboBox = new JComboBox(Constants.ENDIANESS_TYPE);
    endianComboBox.setEnabled(false);
    endianComboBox.addItemListener(itemListener);

    endianChkBox = new JCheckBox(Constants.FORCE_ENDIANESS, false);
    endianChkBox.addItemListener(itemListener);
    endianChkBox.addActionListener( new CheckBoxAction(this, endianComboBox));
    gridbag.setConstraints(endianChkBox, c);
    endianPanel.add(endianChkBox);

    gridbag.setConstraints(endianComboBox, c);
    endianPanel.add(endianComboBox);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1;
    //Add a placeholder
    JLabel tmp = new JLabel();
    gridbag.setConstraints(tmp, c);
    endianPanel.add(tmp);

    panel2.add(endianPanel);
    //End homogenous/heterogeneous HW

    //Begin compress names
    compressNamesChkbox = new JCheckBox(Constants.COMPRESS_NAMES, false);
    compressNamesChkbox.addItemListener(itemListener);
//    panel3.add(compressNamesChkbox);
    panel4.add(compressNamesChkbox);
    //End compress names

    //Begin marshal/unmarshal code
    marshalButtons = new ButtonGroup();
    JRadioButton inlineMarshal = new JRadioButton(Constants.INLINE_MARSHAL, true);
    inlineMarshal.addItemListener(itemListener);
    marshalButtons.add(inlineMarshal);
    JRadioButton libMarshal = new JRadioButton(Constants.LIB_MARSHAL, false);
    libMarshal.addItemListener(itemListener);
    marshalButtons.add(libMarshal);
    panel4.add(inlineMarshal);
    panel4.add(libMarshal);
    //End marshal/unmarshal code

    //--Begin CLDC section
    cldcChkBox = new JCheckBox(Constants.FORCE_CLDC, false);
    cldcChkBox.addItemListener(itemListener);
    panel6.add(cldcChkBox);

    //--Begin Debug section
    JPanel debugPanel = new JPanel();
    c.fill = GridBagConstraints.BOTH;
    c.gridwidth = 1;
    c.weightx = 0;
    debugPanel.setLayout(gridbag);

    debugChkBox = new JCheckBox(Constants.DEBUG, false);
    debugChkBox.addItemListener(itemListener);
    gridbag.setConstraints(debugChkBox, c);
    debugPanel.add(debugChkBox);
    //-- Debug level
    debugLevel = new JTextField("0", 2);
    debugLevel.getDocument().addDocumentListener(docListener);
    gridbag.setConstraints(debugLevel, c);
    debugPanel.add(debugLevel);
    debugPanel.add(new JLabel(Constants.DEBUG_LEVEL));
    c.weightx = 1;
    JLabel space = new JLabel(" ");
    gridbag.setConstraints(space, c);
    debugPanel.add(space);    // for good looks?
    //-- Debug memory levels (instrumentation in Client/Server/etc
    debugMemoryChkBox = new JCheckBox(Constants.DEBUG_MEMORY, false);
    debugMemoryChkBox.addItemListener(itemListener);
    gridbag.setConstraints(debugMemoryChkBox, c);
    debugPanel.add(debugMemoryChkBox);
    JLabel space2 = new JLabel(" ");
    gridbag.setConstraints(space2, c);
    debugPanel.add(space2);    // for good looks?
    //-- Debug timing values (instrumentation in Client/Server/ORB/etc
    debugTimingChkBox = new JCheckBox(Constants.DEBUG_TIMING, false);
    debugTimingChkBox.addItemListener(itemListener);
    gridbag.setConstraints(debugTimingChkBox, c);
    debugPanel.add(debugTimingChkBox);
    
    panel6.add(debugPanel);
    //--End DEBUG section

    //Add panels to the main panel
    panel.add(panel1);
    panel.add(panel2);
//    panel.add(panel3);    
    panel.add(panel4);
//    panel.add(panel5);
    panel.add(panel6);
    return panel;
  }
  
  /*********************************************************
  * Creates panel 5
  * Contains options for Fault Tolerance properties
  *********************************************************/

        private JPanel createPanelFaultTol() {

            JPanel panel = new JPanel();

            jSPFaultTol = new javax.swing.JScrollPane();
            jPFaultTol1 = new javax.swing.JPanel();
            jPFaultTol2 = new javax.swing.JPanel();
            chkTemporal = new javax.swing.JCheckBox();
            jPFaultTol3 = new javax.swing.JPanel();
            jLblTemporal = new javax.swing.JLabel();
            jTFNumTemporal = new javax.swing.JTextField();
            jPFaultTol4 = new javax.swing.JPanel();
            chkSpatial = new javax.swing.JCheckBox();
            jPFaultTol5 = new javax.swing.JPanel();
            chkFTSTCP = new javax.swing.JCheckBox();
            chkFTSUnUDP = new javax.swing.JCheckBox();
            chkFTSGBNUDP = new javax.swing.JCheckBox();
            chkFTSSNWUDP = new javax.swing.JCheckBox();
            chkFTSSerial = new javax.swing.JCheckBox();
            chkFTSOneWire = new javax.swing.JCheckBox();
            jPFaultTol6 = new javax.swing.JPanel();
            chkGroupCom = new javax.swing.JCheckBox();
            jPFaultTol7 = new javax.swing.JPanel();
            jPFaultTol8 = new javax.swing.JPanel();
            jLblUniform = new javax.swing.JLabel();
            jRBUniform = new javax.swing.JRadioButton();
            jRBNonUniform = new javax.swing.JRadioButton();
            jPFaultTol9 = new javax.swing.JPanel();
            jLblOrdering = new javax.swing.JLabel();
            jRBOrderNone = new javax.swing.JRadioButton();
            jRBOrderFIFO = new javax.swing.JRadioButton();
            jRBOrderCausal = new javax.swing.JRadioButton();
            jRBOrderTotal = new javax.swing.JRadioButton();
            jPFaultTol10 = new javax.swing.JPanel();
            jLblTimeOut = new javax.swing.JLabel();
            jTFTimeOut = new javax.swing.JTextField();
            jPFaultTol11 = new javax.swing.JPanel();
            jLblGCTransport = new javax.swing.JLabel();
            jRBGroupTransTCP = new javax.swing.JRadioButton();
            jRBGroupTransUDP = new javax.swing.JRadioButton();
            jPFaultTol12 = new javax.swing.JPanel();
            chkValue = new javax.swing.JCheckBox();

            panel.setLayout(new java.awt.BorderLayout());

            jPFaultTol1.setLayout(new java.awt.GridLayout(4, 1));

            jPFaultTol2.setLayout(new java.awt.GridLayout(2, 1));

            jPFaultTol2.setBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED));
            chkTemporal.setText("Temporal Redundancy");
            chkTemporal.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    chkTemporalActionPerformed(evt);
                }
            });

            jPFaultTol2.add(chkTemporal);

            jLblTemporal.setText("Number of Retransmits:");
            jPFaultTol3.add(jLblTemporal);

            jTFNumTemporal.setText("1");
            jTFNumTemporal.setPreferredSize(new java.awt.Dimension(50, 19));
            jTFNumTemporal.setEnabled(false);
            jPFaultTol3.add(jTFNumTemporal);

            jPFaultTol2.add(jPFaultTol3);

            jPFaultTol1.add(jPFaultTol2);

            jPFaultTol4.setLayout(new java.awt.GridLayout(2, 1));

            jPFaultTol4.setBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED));
            chkSpatial.setText("Spatial Redundancy");
            chkSpatial.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    chkSpatialActionPerformed(evt);
                }
            });

            jPFaultTol4.add(chkSpatial);

            jPFaultTol5.setLayout(new java.awt.GridLayout(2, 3));

            chkFTSTCP.setText("TCP / IP");
            chkFTSTCP.setEnabled(false);
            jPFaultTol5.add(chkFTSTCP);

            chkFTSUnUDP.setText("Unreliable UDP");
            chkFTSUnUDP.setEnabled(false);
            chkFTSUnUDP.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    chkFTSUnUDPActionPerformed(evt);
                }
            });

            jPFaultTol5.add(chkFTSUnUDP);

            chkFTSGBNUDP.setText("Go Back N UDP");
            chkFTSGBNUDP.setEnabled(false);
            chkFTSGBNUDP.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    chkFTSGBNUDPActionPerformed(evt);
                }
            });

            jPFaultTol5.add(chkFTSGBNUDP);

            chkFTSSNWUDP.setText("Stop-N-Wait UDP");
            chkFTSSNWUDP.setEnabled(false);
            chkFTSSNWUDP.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    chkFTSSNWUDPActionPerformed(evt);
                }
            });

            jPFaultTol5.add(chkFTSSNWUDP);

            chkFTSSerial.setText("Serial");
            chkFTSSerial.setEnabled(false);
            jPFaultTol5.add(chkFTSSerial);

            chkFTSOneWire.setText("1-Wire");
            chkFTSOneWire.setEnabled(false);
            jPFaultTol5.add(chkFTSOneWire);

            jPFaultTol4.add(jPFaultTol5);

            jPFaultTol1.add(jPFaultTol4);

            jPFaultTol6.setLayout(new java.awt.GridLayout(2, 1));

            jPFaultTol6.setBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED));
            chkGroupCom.setText("Group Communications (Server)");
            chkGroupCom.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    chkGroupComActionPerformed(evt);
                }
            });

            jPFaultTol6.add(chkGroupCom);

            jPFaultTol7.setLayout(new java.awt.GridLayout(4, 1));

            jPFaultTol8.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 10, 0));

            jLblUniform.setText("Uniformity:");
            jPFaultTol8.add(jLblUniform);

            jRBUniform.setText("Uniform");
            jRBUniform.setEnabled(false);
            jRBUniform.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jRBUniformActionPerformed(evt);
                }
            });

            jPFaultTol8.add(jRBUniform);

            jRBNonUniform.setText("Non-Uniform");
            jRBNonUniform.setEnabled(false);
            jRBNonUniform.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jRBNonUniformActionPerformed(evt);
                }
            });

            jPFaultTol8.add(jRBNonUniform);

            jPFaultTol7.add(jPFaultTol8);

            jPFaultTol9.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 10, 0));

            jLblOrdering.setText("Ordering:");
            jPFaultTol9.add(jLblOrdering);

            jRBOrderNone.setText("None");
            jRBOrderNone.setEnabled(false);
            jRBOrderNone.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jRBOrderNoneActionPerformed(evt);
                }
            });

            jPFaultTol9.add(jRBOrderNone);

            jRBOrderFIFO.setText("FIFO");
            jRBOrderFIFO.setEnabled(false);
            jRBOrderFIFO.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jRBOrderFIFOActionPerformed(evt);
                }
            });

            jPFaultTol9.add(jRBOrderFIFO);

            jRBOrderCausal.setText("Causal");
            jRBOrderCausal.setEnabled(false);
            jRBOrderCausal.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jRBOrderCausalActionPerformed(evt);
                }
            });

            jPFaultTol9.add(jRBOrderCausal);

            jRBOrderTotal.setText("Total");
            jRBOrderTotal.setEnabled(false);
            jRBOrderTotal.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jRBOrderTotalActionPerformed(evt);
                }
            });

            jPFaultTol9.add(jRBOrderTotal);

            jPFaultTol7.add(jPFaultTol9);

            jPFaultTol10.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 10, 0));

            jLblTimeOut.setText("Failure Detection Time Out (ms):");
            jPFaultTol10.add(jLblTimeOut);

            jTFTimeOut.setText("0");
            jTFTimeOut.setMinimumSize(new java.awt.Dimension(63, 20));
            jTFTimeOut.setPreferredSize(new java.awt.Dimension(50, 19));
            jTFTimeOut.setEnabled(false);
            jPFaultTol10.add(jTFTimeOut);

            jPFaultTol7.add(jPFaultTol10);

            jPFaultTol11.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 10, 0));

            jLblGCTransport.setText("Group Comm. Transport:");
            jPFaultTol11.add(jLblGCTransport);

            jRBGroupTransTCP.setText("TCP / IP");
            jRBGroupTransTCP.setEnabled(false);
            jRBGroupTransTCP.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jRBGroupTransTCPActionPerformed(evt);
                }
            });

            jPFaultTol11.add(jRBGroupTransTCP);

            jRBGroupTransUDP.setText("UDP");
            jRBGroupTransUDP.setEnabled(false);
            jRBGroupTransUDP.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jRBGroupTransUDPActionPerformed(evt);
                }
            });

            jPFaultTol11.add(jRBGroupTransUDP);

            jPFaultTol7.add(jPFaultTol11);

            jPFaultTol6.add(jPFaultTol7);

            jPFaultTol1.add(jPFaultTol6);

            jPFaultTol12.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 0, 10));

            jPFaultTol12.setBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED));
            chkValue.setText("Value Redundancy");
            jPFaultTol12.add(chkValue);

            jPFaultTol1.add(jPFaultTol12);

            jSPFaultTol.setViewportView(jPFaultTol1);

            panel.add(jSPFaultTol, java.awt.BorderLayout.CENTER);
            
            return panel;

        }

        private void jRBGroupTransUDPActionPerformed(java.awt.event.ActionEvent evt) {
            // here we need to make sure that at least one option is selected
            if(jRBGroupTransUDP.isSelected()) {
                // make sure thatthe other option is not selected
                if(jRBGroupTransTCP.isSelected()) {
                    jRBGroupTransTCP.setSelected(false);
                }
            } else {
                // make sure that the other option is selected
                if(!(jRBGroupTransTCP.isSelected())) {
                    jRBGroupTransTCP.setSelected(true);
                }
            }
        }

        private void jRBGroupTransTCPActionPerformed(java.awt.event.ActionEvent evt) {
            // here we need to make sure that at least one option is selected
            if(jRBGroupTransTCP.isSelected()) {
                // make sure that the other choice is not selected
                if(jRBGroupTransUDP.isSelected()) {
                    jRBGroupTransUDP.setSelected(false);
                }
            } else {
                // make sure that the other option is selected
                if(!(jRBGroupTransUDP.isSelected())) {
                    jRBGroupTransUDP.setSelected(true);
                }
            }
        }

        private void jRBOrderTotalActionPerformed(java.awt.event.ActionEvent evt) {
            // here we need to make sure that only one ordering option is selected
            if(jRBOrderTotal.isSelected()) {
                // make sure that no other option is selected
                if(jRBOrderNone.isSelected()) {
                    jRBOrderNone.setSelected(false);
                }
                if(jRBOrderFIFO.isSelected()) {
                    jRBOrderFIFO.setSelected(false);
                }
                if(jRBOrderCausal.isSelected()) {
                    jRBOrderCausal.setSelected(false);
                }
            } else {
                // here we need to make sure at least one option is selected
                if(!(jRBOrderNone.isSelected())) {
                    jRBOrderNone.setSelected(true);
                }
                if(jRBOrderFIFO.isSelected()) {
                    jRBOrderFIFO.setSelected(false);
                }
                if(jRBOrderCausal.isSelected()) {
                    jRBOrderCausal.setSelected(false);
                }
            }
        }

        private void jRBOrderCausalActionPerformed(java.awt.event.ActionEvent evt) {
            // here we need to make sure that only one ordering option is selected
            if(jRBOrderCausal.isSelected()) {
                // make sure that no other option is selected
                if(jRBOrderNone.isSelected()) {
                    jRBOrderNone.setSelected(false);
                }
                if(jRBOrderFIFO.isSelected()) {
                    jRBOrderFIFO.setSelected(false);
                }
                if(jRBOrderTotal.isSelected()) {
                    jRBOrderTotal.setSelected(false);
                }
            } else {
                // here we need to make sure at least one option is selected
                if(!(jRBOrderTotal.isSelected())) {
                    jRBOrderTotal.setSelected(true);
                }
                if(jRBOrderNone.isSelected()) {
                    jRBOrderNone.setSelected(false);
                }
                if(jRBOrderFIFO.isSelected()) {
                    jRBOrderFIFO.setSelected(false);
                }
            }
        }

        private void jRBOrderFIFOActionPerformed(java.awt.event.ActionEvent evt) {
            // here we need to make sure that only one ordering option is selected
            if(jRBOrderFIFO.isSelected()) {
                // make sure that no other option is selected
                if(jRBOrderNone.isSelected()) {
                    jRBOrderNone.setSelected(false);
                }
                if(jRBOrderCausal.isSelected()) {
                    jRBOrderCausal.setSelected(false);
                }
                if(jRBOrderTotal.isSelected()) {
                    jRBOrderTotal.setSelected(false);
                }
            } else {
                // here we need to make sure that at least on option is selected
                if(!(jRBOrderCausal.isSelected())) {
                    jRBOrderCausal.setSelected(true);
                }
                if(jRBOrderNone.isSelected()) {
                    jRBOrderNone.setSelected(false);
                }
                if(jRBOrderTotal.isSelected()) {
                    jRBOrderTotal.setSelected(false);
                }
            }
        }

        private void jRBOrderNoneActionPerformed(java.awt.event.ActionEvent evt) {
            // here we need to make sure that only one ordering option is selected
            if(jRBOrderNone.isSelected()) {
                // make sure that no other option is selected
                if(jRBOrderFIFO.isSelected()) {
                    jRBOrderFIFO.setSelected(false);
                }
                if(jRBOrderCausal.isSelected()) {
                    jRBOrderCausal.setSelected(false);
                }
                if(jRBOrderTotal.isSelected()) {
                    jRBOrderTotal.setSelected(false);
                }
            } else {
                // here we need to make sure that at least one option is selected
                if(!(jRBOrderFIFO.isSelected())) {
                    jRBOrderFIFO.setSelected(true);
                }
                if(jRBOrderCausal.isSelected()) {
                    jRBOrderCausal.setSelected(false);
                }
                if(jRBOrderTotal.isSelected()) {
                    jRBOrderTotal.setSelected(false);
                }
            }

        }

        private void jRBNonUniformActionPerformed(java.awt.event.ActionEvent evt) {
            // here we need to make sure that the two option buttons are 
            //  not active at the same time
            if(jRBNonUniform.isSelected()) {
                // here we need to make sure that the other is not selected
                if(jRBUniform.isSelected()) {
                    jRBUniform.setSelected(false);
                }
            } else {
                // here we need to make sure that the other is selected
                if(!(jRBUniform.isSelected())) {
                    jRBUniform.setSelected(true);
                }
            }
        }

        private void jRBUniformActionPerformed(java.awt.event.ActionEvent evt) {
            // here we need to make sure that the two option buttons
            //  are not active at the same time
            if(jRBUniform.isSelected()) {
                // here we need to make sure that the other is not selected
                if(jRBNonUniform.isSelected()) {
                    jRBNonUniform.setSelected(false);
                }
            } else {
                // here we need to make sure that the other is selected
                if(!(jRBNonUniform.isSelected())) {
                    jRBNonUniform.setSelected(true);
                }
            }
        }

        private void chkGroupComActionPerformed(java.awt.event.ActionEvent evt) {
            // check to see if the checkbox was checked or not
            if(chkGroupCom.isSelected()) {
                // here we need to make sure that all the related controls are
                //  enabled
                jRBUniform.setEnabled(true);
                jRBNonUniform.setEnabled(true);
                jRBOrderNone.setEnabled(true);
                jRBOrderFIFO.setEnabled(true);
                jRBOrderCausal.setEnabled(true);
                jRBOrderTotal.setEnabled(true);
                jTFTimeOut.setEnabled(true);
                jRBGroupTransTCP.setEnabled(true);
                jRBGroupTransUDP.setEnabled(true);
            } else {
                // here we need to disable all the related controls
                jRBUniform.setEnabled(false);
                jRBNonUniform.setEnabled(false);
                jRBOrderNone.setEnabled(false);
                jRBOrderFIFO.setEnabled(false);
                jRBOrderCausal.setEnabled(false);
                jRBOrderTotal.setEnabled(false);
                jTFTimeOut.setEnabled(false);
                jRBGroupTransTCP.setEnabled(false);
                jRBGroupTransUDP.setEnabled(false);
            }
        }

        private void chkTemporalActionPerformed(java.awt.event.ActionEvent evt) {
            if(chkTemporal.isSelected()) {
                jTFNumTemporal.setEnabled(true);
            } else {
                jTFNumTemporal.setEnabled(false);
            }
        }

        private void chkSpatialActionPerformed(java.awt.event.ActionEvent evt) {
            if(chkSpatial.isSelected()) {
                // here we need to make sure all the check boxes are enabled
                chkFTSSerial.setEnabled(true);
                chkFTSOneWire.setEnabled(true);
                chkFTSSNWUDP.setEnabled(true);
                chkFTSGBNUDP.setEnabled(true);
                chkFTSUnUDP.setEnabled(true);
                chkFTSTCP.setEnabled(true);
                // here we will need to check what the primary communication transport is
            } else {
                // here we need to disable all the checkboxes
                chkFTSSerial.setEnabled(false);
                chkFTSOneWire.setEnabled(false);
                chkFTSSNWUDP.setEnabled(false);
                chkFTSGBNUDP.setEnabled(false);
                chkFTSUnUDP.setEnabled(false);
                chkFTSTCP.setEnabled(false);
            }
        }

        private void chkFTSGBNUDPActionPerformed(java.awt.event.ActionEvent evt) {
            // here we need to make sure that the other two UDP boxes are not checked
            chkFTSUnUDP.setSelected(false);
            chkFTSSNWUDP.setSelected(false);
        }

        private void chkFTSUnUDPActionPerformed(java.awt.event.ActionEvent evt) {
            // here we need to make sure that the other two UDP boxes are not checked
            chkFTSGBNUDP.setSelected(false);
            chkFTSSNWUDP.setSelected(false);
        }

        private void chkFTSSNWUDPActionPerformed(java.awt.event.ActionEvent evt) {
            // here we need to make sure that the other two UDP boxes are not checked
            chkFTSGBNUDP.setSelected(false);
            chkFTSUnUDP.setSelected(false);
        }

  /*********************************************************
  * Reads the values of the controls in the panels and
  * updates the properties
  *********************************************************/
  public void updateProperties()
  {
    //Start Tab 1
    //Read data types checkbox value and set property to true or false
    updateCheckboxProperties(primTypesChkBox, Constants.PRIMITIVE_TYPE, Constants.NO_PRIMITIVE_TYPES, null);
    updateCheckboxProperties(complexTypesChkBox, Constants.COMPLEX_TYPE, Constants.NO_COMPLEX_TYPES, null);
    updateCheckboxProperties(exceptionTypesChkBox, Constants.EXCEPTION_TYPE, Constants.NO_EXCEPTION_TYPES, null);
    //End Tab 1

    //Start Tab 2
    //Read checkbox value and set QoS properties to true or false
    updateCheckboxProperty(secChkBox, Constants.SECURITY);
    updateCheckboxProperty(secMDChkBox, Constants.SECURITY_MD);
    updateCheckboxProperty(secMACChkBox, Constants.SECURITY_MAC);
    updateCheckboxProperty(secAuditChkBox, Constants.SECURITY_AUDIT);
    updateCheckboxProperty(RTChkBox, Constants.REALTIME);
    updateCheckboxProperty(NetChkBox, Constants.NETWORK);

    //Read radiobutton value and set each property to true or false
    updateRadioButtonProperty(Constants.SEC_CIPHER, secCipherButtonGroup.getElements());
    updateRadioButtonProperty(Constants.SEC_CIPHER_MODE, secCipherModeButtonGroup.getElements());
    updateRadioButtonProperty(Constants.SEC_CIPHER_PADDING, secCipherPaddingButtonGroup.getElements());
    //Set the Cipher KeyLength property from the text field
    try {
      int temp = (new Integer(secCipherKeyLenText.getText())).intValue();
      MQCProperties.setProperty(Constants.SEC_CIPHER_KEY_LEN, secCipherKeyLenText.getText());
    }
    catch(NumberFormatException ex)    {
      System.out.println("Please specify an integer for the Cipher Key Length");
      MQCProperties.setProperty(Constants.SEC_CIPHER_KEY_LEN, Constants.SEC_CIPHER_KEY_DEFS[0]);
    }
    
    // Message Digest
    updateRadioButtonProperty(Constants.SEC_MD_LABEL, secMDButtonGroup.getElements());
    
    // Set the MAC Key length property from the text field
    updateRadioButtonProperty(Constants.SEC_MAC_LABEL, secMACButtonGroup.getElements());
    try {
      int temp = (new Integer(secMACKeyLenText.getText())).intValue();
      MQCProperties.setProperty(Constants.SEC_MAC_KEY_LEN, secMACKeyLenText.getText());
    }
    catch(NumberFormatException ex) {
      System.out.println("Please specify an integer for the MAC Key Length");
      MQCProperties.setProperty(Constants.SEC_MAC_KEY_LEN, Constants.SEC_MAC_KEY_DEFS[0]);
    }
    
    // Audit
    updateRadioButtonProperty(Constants.SEC_AUDIT_LABEL, secAuditButtonGroup.getElements());
    try {
      int temp = (new Integer(secAuditLogLenText.getText())).intValue();
      MQCProperties.setProperty(Constants.SEC_AUDIT_LOG_LEN, secAuditLogLenText.getText());
    }
    catch(NumberFormatException ex) {
      System.out.println("Please specify an integer for the Audit Log Length");
      MQCProperties.setProperty(Constants.SEC_AUDIT_LOG_LEN, Constants.SEC_AUDIT_LOG_DEFS[0]);
    }

    updateRadioButtonProperties(realtimeButtons.getElements());

    //Read checkbox value and set Network QoS properties to true or false
    Enumeration e = netQoSButtons.elements();
    while(e.hasMoreElements())
    {
      JCheckBox chkBox = (JCheckBox) e.nextElement();
      MQCProperties.setProperty(chkBox.getText(), (new Boolean(chkBox.isSelected()).toString()));
    }

    //Set the bandwidth property from the text field
    try
    {
      int temp = (new Integer(bandwidth.getText())).intValue();
      MQCProperties.setProperty(Constants.NET_GUARANTEED_BW_NUM, bandwidth.getText());
    }
    catch(NumberFormatException ex)
    {
      System.out.println("Please specify an integer for network bandwidth");
      MQCProperties.setProperty(Constants.NET_GUARANTEED_BW_NUM, "0");
    }
    //End Tab 2
        
    //Start Tab 3
    //Read GIOP and Transport radiobutton value and set each property to true or false
    updateRadioButtonProperties(clientGIOPButtons.getElements());
    updateRadioButtonProperties(clientTRButtons.getElements());

    //Read checkbox value and set transport properties to true or false
    updateCheckboxProperties(serverTRChkBox, Constants.TRANSPORT_TYPE, Constants.NO_TRANSPORTS, Constants.SERVER_HDR);
    //Read checkbox value and set protocol properties to true or false
    updateCheckboxProperties(serverGIOPChkBox, Constants.PROTOCOL_TYPE, Constants.NO_PROTOCOLS, Constants.SERVER_HDR);

    //End Tab 3

    //Start Tab 4
    //Set the message payload property from the text field
    try
    {
      int temp = (new Integer(messagePayload.getText())).intValue();
      MQCProperties.setProperty(Constants.MSG_PAYLOAD, messagePayload.getText());
    }
    catch(NumberFormatException ex)
    {
      System.out.println("Please specify an integer for message payload");
      MQCProperties.setProperty(Constants.MSG_PAYLOAD, "0");
    }
  
    //Set the number of interfaces property from the text field
    String number = noInterfaces.getText();
    if(number.equals(Constants.NO_RESTRICTION))
      MQCProperties.setProperty(Constants.NO_INTERFACES, "0");    
    else
      MQCProperties.setProperty(Constants.NO_INTERFACES, number);    

    //Set the number of methods property from the text field
    number = noMethods.getText();
    if(number.equals(Constants.NO_RESTRICTION))
      MQCProperties.setProperty(Constants.NO_METHODS, "0");    
    else
      MQCProperties.setProperty(Constants.NO_METHODS, number);    

    updateRadioButtonProperties(hwButtons.getElements());

    //Read the force endianess checkbox and set the property to true or false.
    MQCProperties.setProperty(Constants.FORCE_ENDIANESS, 
                           (new Boolean(endianChkBox.isSelected()).toString()));

    //Set the endianess property
    int index = endianComboBox.getSelectedIndex();
    MQCProperties.setProperty(Constants.ENDIANESS, String.valueOf(index));
    
    //Read the compress names checkbox and set the property to true or false.
    MQCProperties.setProperty(Constants.COMPRESS_NAMES, 
                    (new Boolean(compressNamesChkbox.isSelected()).toString()));

    updateRadioButtonProperties(marshalButtons.getElements());

    //Set the CLDC property
    MQCProperties.setProperty(Constants.FORCE_CLDC,
                        (new Boolean(cldcChkBox.isSelected()).toString()));

    //Set the Debug properties
    MQCProperties.setProperty(Constants.DEBUG,
                        (new Boolean(debugChkBox.isSelected()).toString()));
    try
    {
      int temp = (new Integer(debugLevel.getText())).intValue();
      MQCProperties.setProperty(Constants.DEBUG_LEVEL, debugLevel.getText());
    }
    catch(NumberFormatException ex)
    {
      System.out.println("Please specify an integer for the Debug Level");
      MQCProperties.setProperty(Constants.DEBUG_LEVEL, "0");
    }
    MQCProperties.setProperty(Constants.DEBUG_MEMORY,
                        (new Boolean(debugMemoryChkBox.isSelected()).toString()));
    MQCProperties.setProperty(Constants.DEBUG_TIMING,
                        (new Boolean(debugTimingChkBox.isSelected()).toString()));
    //End Tab 4
    
    // Start Tab 5
    if(chkTemporal.isSelected()) {
      MQCProperties.setProperty("enableTemporal","true");
      String strTemporal = jTFNumTemporal.getText();
      MQCProperties.setProperty("temporal",strTemporal);
    } else {
      MQCProperties.setProperty("enableTemporal","false");
      String strTemporal = "1";
      MQCProperties.setProperty("temporal",strTemporal);
    }
    if(chkSpatial.isSelected()) {
      MQCProperties.setProperty("enableSpatial","true");
      // next load the properties for all the related check boxes
      MQCProperties.setProperty("ftsTCP",(new Boolean(chkFTSTCP.isSelected()).toString()));
      MQCProperties.setProperty("ftsUnUDP",(new Boolean(chkFTSUnUDP.isSelected()).toString()));
      MQCProperties.setProperty("ftsGBNUDP",(new Boolean(chkFTSGBNUDP.isSelected()).toString()));
      MQCProperties.setProperty("ftsSNWUDP",(new Boolean(chkFTSSNWUDP.isSelected()).toString()));
      MQCProperties.setProperty("ftsSerial",(new Boolean(chkFTSSerial.isSelected()).toString()));
      MQCProperties.setProperty("ftsOneWire",(new Boolean(chkFTSOneWire.isSelected()).toString()));
    } else {
      MQCProperties.setProperty("enableSpatial","false");
      // next load the properties for all the related check boxes
      MQCProperties.setProperty("ftsTCP","false");
      MQCProperties.setProperty("ftsUnUDP","false");
      MQCProperties.setProperty("ftsGBNUDP","false");
      MQCProperties.setProperty("ftsSNWUDP","false");
      MQCProperties.setProperty("ftsSerial","false");
      MQCProperties.setProperty("ftsOneWire","false");
    }
                
    if(chkGroupCom.isSelected()) {
      // here the group communication mechanism has been selected
      MQCProperties.setProperty("enableGroupCom","true");
    } else {
      MQCProperties.setProperty("enableGroupCom","false");
    }
    // next load all the properties for the related options
    if(jRBUniform.isSelected()) {
      MQCProperties.setProperty("ftGCUniform","uniform");
    } else {
      MQCProperties.setProperty("ftGCUniform","nonuniform");
    }
    String strOrdering = "none";
    if(jRBOrderNone.isSelected()) {
      strOrdering = "none";
    } else if(jRBOrderFIFO.isSelected()) {
      strOrdering = "fifo";
    } else if(jRBOrderCausal.isSelected()) {
      strOrdering = "causal";
    } else if(jRBOrderTotal.isSelected()) {
      strOrdering = "total";
    }
    MQCProperties.setProperty("ftGCOrder",strOrdering);
    MQCProperties.setProperty("ftGCTimeOut",jTFTimeOut.getText());
    if(jRBGroupTransUDP.isSelected()) {
      MQCProperties.setProperty("ftGCTrans","udp");
    } else {
      MQCProperties.setProperty("ftGCTrans","tcp");
    }

    if(chkValue.isSelected()) {
      MQCProperties.setProperty("enableValue","true");
    } else {
      MQCProperties.setProperty("enableValue","false");
    }

    // End Tab 5
    
    /*-------------------
     * GIOP Version HACK 
     *-------------------
     */
    MQCProperties.setProperty(Constants.GIOP_V10_STR, "false");
    MQCProperties.setProperty(Constants.GIOP_V11_STR, "false");
    MQCProperties.setProperty(Constants.GIOP_V12_STR, "true");
    MQCProperties.setProperty(Constants.GIOP_V13_STR, "false");
  }    

  /*********************************************************
  * Iterates through the radiobutton group and updates
  * the properties.
  *
  * @param Enumeration The list of radiobuttons to be saved
  *                     in the property list
  *********************************************************/
  private void updateRadioButtonProperties(Enumeration e)
  {
    JRadioButton button = null;
    while(e.hasMoreElements())
    {
      button = (JRadioButton) e.nextElement();
      if(button.isSelected())
      {
        MQCProperties.setProperty(button.getText(), "true");
      }
      else
      {
        MQCProperties.setProperty(button.getText(), "false");
      }
    }
  }

  /*********************************************************
  * Sets a property to the text value of the 'true' radio button
  *
  * @param propName  Name of the property to be set
  * @param e         enumeration of radio buttons
  *********************************************************/
  private void updateRadioButtonProperty(String propName, Enumeration e)
  {
    JRadioButton button = null;
    while(e.hasMoreElements()) {
      button = (JRadioButton) e.nextElement();
      if(button.isSelected()) {
        MQCProperties.setProperty(propName, button.getText());
        return;
      }
    }
  }

  /*********************************************************
  * Update the property associated with a single checkbox.
  *
  * @param JCheckBox the checkbox which should be read and saved
  * @param String    the checkbox's property name
  *********************************************************/
  private void updateCheckboxProperty(JCheckBox chkBox, String property)
  {
    Boolean value = new Boolean(chkBox.isSelected());
    MQCProperties.setProperty(property, value.toString());
  }
  
  /*********************************************************
  * Iterates through checkboxes and updates the properties.
  *
  * @param JCheckBox A list of checkboxes to be read and
  *                   values saved to the properties.
  * @param String A list of property names
  * @param int The number of check boxes
  * @param String The prefix for the property name
  *********************************************************/
  private void updateCheckboxProperties(JCheckBox chkBox[], String types[], int no, String prefix)
  {
    Boolean value = null;
    if(prefix == null)
      prefix = "";
    else
      prefix += "_";
    for(int i = 0; i < no; i++)
    {
      value = new Boolean(chkBox[i].isSelected());
      MQCProperties.setProperty(prefix + types[i], value.toString());
    }
  }
  
  /*********************************************************
  * Populates the controls with the saved data
  *********************************************************/
  public void populate()
  {
    //Start TAB 1
    //Read in the data types from the properties and populate the checkboxes
    populateCheckboxes(primTypesChkBox, Constants.PRIMITIVE_TYPE, Constants.NO_PRIMITIVE_TYPES, false, null);
    populateCheckboxes(complexTypesChkBox, Constants.COMPLEX_TYPE, Constants.NO_COMPLEX_TYPES, false, null);
    populateCheckboxes(exceptionTypesChkBox, Constants.EXCEPTION_TYPE, Constants.NO_EXCEPTION_TYPES, false, null);
    //End TAB 1


    //Start TAB 2
    //Read in the QoS values from the properties and populate the checkboxes
    populateCheckbox(secChkBox, Constants.SECURITY, false);
    populateCheckbox(secMDChkBox, Constants.SECURITY_MD, false);
    populateCheckbox(secMACChkBox, Constants.SECURITY_MAC, false);
    populateCheckbox(secAuditChkBox, Constants.SECURITY_AUDIT, false);
    populateCheckbox(RTChkBox, Constants.REALTIME, false);
    populateCheckbox(NetChkBox, Constants.NETWORK, false);
    
    //Read in the QoS values from the properties and populate the radio buttons
    populateRadioButtons(Constants.SEC_CIPHER, secCipherButtonGroup.getElements());
    populateRadioButtons(Constants.SEC_CIPHER_MODE, secCipherModeButtonGroup.getElements());
    populateRadioButtons(Constants.SEC_CIPHER_PADDING, secCipherPaddingButtonGroup.getElements());
    secCipherKeyLenText.setText(MQCProperties.getProperty(Constants.SEC_CIPHER_KEY_LEN));        
    populateRadioButtons(Constants.SEC_MD_LABEL, secMDButtonGroup.getElements());
    populateRadioButtons(Constants.SEC_MAC_LABEL, secMACButtonGroup.getElements());
    secMACKeyLenText.setText(MQCProperties.getProperty(Constants.SEC_MAC_KEY_LEN));        
    populateRadioButtons(Constants.SEC_AUDIT_LABEL, secAuditButtonGroup.getElements());
    secAuditLogLenText.setText(MQCProperties.getProperty(Constants.SEC_AUDIT_LOG_LEN));        
    
    populateRadioButtons(realtimeButtons.getElements());
    
    Boolean value = null;
    //Read in Network QoS properties and populate the checkboxes
    Enumeration e = netQoSButtons.elements();
    while(e.hasMoreElements())
    {
      JCheckBox chkBox = (JCheckBox) e.nextElement();
      value = new Boolean(MQCProperties.getProperty(chkBox.getText()));

      //This is done to enable or disable the subcategories
      chkBox.setSelected(!value.booleanValue());
      chkBox.doClick();
      chkBox.setSelected(value.booleanValue());
    }

    //Read in the network bandwidth property and populate the text field
    bandwidth.setText(MQCProperties.getProperty(Constants.NET_GUARANTEED_BW_NUM));    
    //End TAB 2

    //Start TAB 3
    //Read in the transport properties and populate the radiobuttons
    populateRadioButtons(clientTRButtons.getElements());
    //Read in the GIOP properties and populate the radiobuttons
    populateRadioButtons(clientGIOPButtons.getElements());

    //Read in the transport properties and populate the radiobuttons
    populateCheckboxes(serverTRChkBox, Constants.TRANSPORT_TYPE, Constants.NO_TRANSPORTS, false, Constants.SERVER_HDR);
    //Read in the GIOP properties and populate the radiobuttons
    populateCheckboxes(serverGIOPChkBox, Constants.PROTOCOL_TYPE, Constants.NO_PROTOCOLS, false, Constants.SERVER_HDR);

    //End TAB 3


    //Start TAB 4
    //Read in the message payload property and populate the text field
    messagePayload.setText(MQCProperties.getProperty(Constants.MSG_PAYLOAD));    
  
    //Read in the number of interfaces and populate the text field
    String number = MQCProperties.getProperty(Constants.NO_INTERFACES);
    noInterfaces.setText(number);

    //Read in the number of methods and populate the text field
    number = MQCProperties.getProperty(Constants.NO_METHODS);
    noMethods.setText(number);

    //Read the compress names radio button and populate the check box
    value = new Boolean(MQCProperties.getProperty(Constants.COMPRESS_NAMES));
    compressNamesChkbox.setSelected(value.booleanValue());
    
    //Read in the homogeneous/heterogeneous properties and populate the radio buttons
    populateRadioButtons(hwButtons.getElements());

    //Read the force endianess checkbox and set the property to true or false.
    value = new Boolean(MQCProperties.getProperty(Constants.FORCE_ENDIANESS));
    endianChkBox.setSelected(value.booleanValue());
    endianComboBox.setEnabled(value.booleanValue());
    
    //Set the endianess property
    try
    {
      int index = Integer.parseInt(MQCProperties.getProperty(Constants.ENDIANESS));
      endianComboBox.setSelectedIndex(index);
    }
    catch(Exception ex)
    {
    }
    //Read in the marshal/unmarshal properties and populate the radio buttons
    populateRadioButtons(marshalButtons.getElements());

    // Set the CLDC property
    value = new Boolean(MQCProperties.getProperty(Constants.FORCE_CLDC));
    cldcChkBox.setSelected(value.booleanValue());

    // Set the Debug properties
    value = new Boolean(MQCProperties.getProperty(Constants.DEBUG));
    debugChkBox.setSelected(value.booleanValue());
    debugLevel.setText(MQCProperties.getProperty(Constants.DEBUG_LEVEL));    
    value = new Boolean(MQCProperties.getProperty(Constants.DEBUG_MEMORY));
    debugMemoryChkBox.setSelected(value.booleanValue());
    value = new Boolean(MQCProperties.getProperty(Constants.DEBUG_TIMING));
    debugTimingChkBox.setSelected(value.booleanValue());

    //End TAB 4
    
    //Start TAB 5
    Boolean blnEnableTemporal = new Boolean(MQCProperties.getProperty("enableTemporal"));
    chkTemporal.setSelected(blnEnableTemporal.booleanValue());

    if(chkTemporal.isSelected()) {
      jTFNumTemporal.setEnabled(true);
    } else {
      jTFNumTemporal.setEnabled(false);
    }

    String strTemporal = MQCProperties.getProperty("temporal");
    jTFNumTemporal.setText(strTemporal);


    Boolean blnEnableSpatial = new Boolean(MQCProperties.getProperty("enableSpatial"));
    chkSpatial.setSelected(blnEnableSpatial.booleanValue());

    Boolean blnLoadSpatialChecks = new Boolean(MQCProperties.getProperty("ftsTCP"));
    chkFTSTCP.setSelected(blnLoadSpatialChecks.booleanValue());
    blnLoadSpatialChecks = new Boolean(MQCProperties.getProperty("ftsUnUDP"));
    chkFTSUnUDP.setSelected(blnLoadSpatialChecks.booleanValue());
    blnLoadSpatialChecks = new Boolean(MQCProperties.getProperty("ftsGBNUDP"));
    chkFTSGBNUDP.setSelected(blnLoadSpatialChecks.booleanValue());
    blnLoadSpatialChecks = new Boolean(MQCProperties.getProperty("ftsSNWUDP"));
    chkFTSSNWUDP.setSelected(blnLoadSpatialChecks.booleanValue());
    blnLoadSpatialChecks = new Boolean(MQCProperties.getProperty("ftsSerial"));
    chkFTSSerial.setSelected(blnLoadSpatialChecks.booleanValue());
    blnLoadSpatialChecks = new Boolean(MQCProperties.getProperty("ftsOneWire"));
    chkFTSOneWire.setSelected(blnLoadSpatialChecks.booleanValue());

    Boolean blnEnableGroupCom = new Boolean(MQCProperties.getProperty("enableGroupCom"));
    chkGroupCom.setSelected(blnEnableGroupCom.booleanValue());
    // check the options under GroupCom
    String strUniform = MQCProperties.getProperty("ftGCUniform");
    if(strUniform!=null && strUniform.equalsIgnoreCase("uniform")) {
      jRBUniform.setSelected(true);
    } else {
      jRBNonUniform.setSelected(true);
    }
    String strOrder = MQCProperties.getProperty("ftGCOrder");
    if(strOrder!=null) {
      if(strOrder.equalsIgnoreCase("none")) {
        jRBOrderNone.setSelected(true);
      } else if(strOrder.equalsIgnoreCase("fifo")) {
        jRBOrderFIFO.setSelected(true);
      } else if(strOrder.equalsIgnoreCase("causal")) {
        jRBOrderCausal.setSelected(true);
      } else if(strOrder.equalsIgnoreCase("total")) {
        jRBOrderTotal.setSelected(true);
      }
    }
    String strTimeOut = MQCProperties.getProperty("ftGCTimeOut");
    if(strTimeOut!=null) {
      jTFTimeOut.setText(strTimeOut);
    }
    String strGCTrans = MQCProperties.getProperty("ftGCTrans");
    if(strGCTrans!=null && strGCTrans.equalsIgnoreCase("tcp")) {
      jRBGroupTransTCP.setSelected(true);
    } else {
      jRBGroupTransUDP.setSelected(true);
    }

    Boolean blnEnableValue = new Boolean(MQCProperties.getProperty("enableValue"));
    chkValue.setSelected(blnEnableValue.booleanValue());

    //End TAB 5
    
    /*-------------------
     * GIOP Version HACK
     *-------------------
     */
    // Nothing to do, the GIOP Version has not/will not be added to the GUI
    // This comment is just a reminder for later when the GUI gets updated.
  }

  /*********************************************************
  * Populates the radio button controls
  *
  * @param e  enumeration of the radio buttons to be set
  *********************************************************/
  private void populateRadioButtons(Enumeration e)
  {
    Boolean value = null;
    JRadioButton button = null;
    while(e.hasMoreElements())
    {
      button = (JRadioButton) e.nextElement();
      value = new Boolean(MQCProperties.getProperty(button.getText()));
      button.setSelected(value.booleanValue());
    }
  }

  /*********************************************************
  * Set a radio button group so that the button with the proper text value
  * is set to true
  *
  * @param propName  property name to look up
  * @param e         enumeration of radio buttons
  *********************************************************/
  private void populateRadioButtons(String propName, Enumeration e)
  {
    Boolean value = null;
    JRadioButton button = null;
    String propValue = MQCProperties.getProperty(propName);
    while(e.hasMoreElements()) {
      button = (JRadioButton) e.nextElement();
      if (propValue.equalsIgnoreCase(button.getText())) {
        button.setSelected(true);
        return;
      }
    }
  }

  /*********************************************************
  * Populate the value of a checkbox
  *
  * @param JCheckBox the checkbox to be set
  * @param String    the property name
  * @param boolean   enable/disable checkbox subcategories
  *********************************************************/
  private void populateCheckbox(JCheckBox chkBox, String property, boolean doClick)
  {
    Boolean value = new Boolean(MQCProperties.getProperty(property));
    if(doClick)
    {
      //This is done to enable or disable the subcategories
      chkBox.setSelected(!value.booleanValue());
      chkBox.doClick();
    }
    chkBox.setSelected(value.booleanValue());
  }

  /*********************************************************
  * Iterates through checkboxes and updates
  * the value
  *
  * @param JCheckBox A list of checkboxes to be set
  * @param String A list of property names
  * @param int The number of check boxes
  * @param String The prefix for the property name
  *********************************************************/
  private void populateCheckboxes(JCheckBox chkBox[], String types[], int no, boolean doClick, String prefix)
  {
    Boolean value = null;
    String type = null;
    for(int i = 0; i < no; i++)
    {
      if(prefix != null)
        type = prefix + "_" + types[i];
      else
        type = types[i];
      value = new Boolean(MQCProperties.getProperty(type));
      if(doClick)
      {
        //This is done to enable or disable the subcategories
        chkBox[i].setSelected(!value.booleanValue());
        chkBox[i].doClick();
      }
      chkBox[i].setSelected(value.booleanValue());
    }
  }

  /*********************************************************
  * Sets the check profile button icon.
  *********************************************************/
  public void setCheckProfileIcon(boolean ok)
  {
    if(ok)
    {
      chkProfileBtn.setIcon(chkProfOnIcon);
      chkProfile.setIcon(chkProfOnIcon);
    }
    else
    {
      try {
        chkProfileBtn.setIcon(chkProfOffIcon);
        chkProfile.setIcon(chkProfOffIcon);
      } catch (Exception e) {}
    }
  }
  
  /*********************************************************
  * Load the image from a jar file
  *
  * @param ImageIcon The loaded icon
  *********************************************************/
  public ImageIcon loadImageFromJar(String imageName) 
  {
    ImageIcon image = null;

    try 
    {
      URLClassLoader classLoader = (URLClassLoader) getClass().getClassLoader();
      URL url = classLoader.findResource(imageName);
      image = new ImageIcon(url);
    } 
    catch (Exception e) 
    {
      System.err.println(e);
      System.exit(1);
    }
    return image;
  }   
}
