/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

//include(`Config_macros.m4')

import mqc.*;
import mqc.holders.*;

import java.io.*;

__SECURITY_ORBKEY__(
//import java.security.InvalidKeyException;
import mqc.security.InvalidKeyException;
)

public class Server
{
  public static void main(String[] args)
  {
    __DEBUG_MEMORY__(
    //ShowMemory();
    System.gc();
    )
    
    S_ORB orb = new S_ORB();
    __DEBUG__( System.out.println("ORB created!");)
    
    __SECURITY__(
    System.out.println( "\n*** " + orb.cipherEncrypt.toString() + "  keyLen: __SEC_CIPHER_KEY_LEN__");
    // load the 'user' supplied key into the ORB
    byte[] orbKey = {10,11,12,13,14,15,16,17};
    try {
      orb.initCipherKey(orbKey);
    }
    catch (InvalidKeyException e) {
      System.out.println(e);
    }
    )
    __SECURITY_MD__(
    System.out.println("*** Using Message Digest: __SEC_MD_ALG__");
    )
    __SECURITY_MAC__(
    System.out.println("*** Using MAC: __SEC_MAC_ALG__ (__SEC_MAC_ALG_PARAMS__)");
    // load the 'user' supplied MAC key into the ORB
    byte[] macKey = {20,21,22,23,24,25,26,27};
    try {
      orb.initMacKey(macKey);
    }
    catch (InvalidKeyException e) {
      System.out.println(e);
    }
    )

    __GROUPCOM__(
    // create a new groupcom object
    System.out.println("Creating GroupCom object");
    String strListenPort = args[0];
    String strHost = System.getProperty("host");
    if(strHost == null) {
      strHost = "localhost.localdomain";
    }
    int intListenPort = Integer.parseInt(strListenPort);
    orb.instanciateGroupCom(strHost, intListenPort);

    if (intListenPort != 10000) {
      String grpStr = System.getProperty("grp");
      if (grpStr == null) {
        if (args.length > 1) {
          grpStr = args[1];
        } else {
          grpStr = "localhost.localdomain:10000";
        }
      }
      System.out.println("Using 'grp' = " + grpStr);

      orb.joinGroup(grpStr);

      try {
        Thread.sleep(500);
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    )


    POA rootPOA = new POA(orb, "RootPOA");
    fooImpl test = new fooImpl();

    Object object = rootPOA.servant_to_reference( test );

//    //-- Generate the IOR --
//    String ior      = orb.object_to_string(object);
//    __DEBUG__( System.out.println("object to string");)
//    System.out.println(ior);

    //--Generate the corbaloc
    String corbaloc = orb.object_to_corbaloc(object);
    __DEBUG__( System.out.println("object to corbaloc!");)
    System.out.println(corbaloc);

    __J2SE__(
    try {
//      FileWriter out = new FileWriter(new File("timing.ior"));
//      out.write(ior);
//      out.close();

      FileWriter out = new FileWriter(new File("timing.corbaloc"));
      out.write(corbaloc);
      out.close();
    }
    catch(IOException e)
    {
      System.out.println("Error writing IOR/corbloc");
      return;
    }
    )

    orb.run();
    
    __DEBUG_MEMORY__(
    // Give the ORB time to start running/waiting for client connections
    try {
      Thread.sleep(500);
    } catch (InterruptedException ie) {}
    ShowMemory();
    
    __DEBUG__(
    for (;;) {  // continuous loop of ShowMemory values
      try {
        Thread.sleep(5000);
      } catch (InterruptedException ie) {}
      ShowMemory();
    }
    )
    )
  }
  
  
  __CLDC__(
  static void AllDone() {
    // This function is needed so that a S/W break point can be set
    // that will cause Charade to timeout of its 'run X' command
  }
  )
  
  
  __DEBUG_MEMORY__(
  static Runtime runtime = Runtime.getRuntime();
  
  static void ShowMemory() {
    long tMem, fMem, uMem, fMem1, uMemMax, fMemMax;

    tMem    = runtime.totalMemory();
    fMem    = runtime.freeMemory();
    fMemMax = fMem;
    if (tMem == fMem) {
      // TINI hack, tMem and fMem are reported as equal
      tMem = 334240;  // MAGIC NUMBER appears to be the proper value
    }
    uMem    = tMem - fMem;
    __DEBUG__( System.out.println("s.Memory (t/f/u): \t"+tMem+" \t"+fMem+" \t"+uMem);)

    System.gc();
    
    for (int i = 0;  i < 100;  i++) {
      fMem1 = runtime.freeMemory();
      __DEBUG__( System.out.println("s.fMem: " + fMem1););

      if (fMem1 > fMemMax ) {
        fMemMax = fMem1;
      } else {
        if (i > 1) break;
      }
      try {
        Thread.sleep(500);
      } catch (InterruptedException ie) {}
    }
    uMemMax = tMem - fMemMax;

    System.out.println("@@@ s.Memory (t/f/u/du): \t"
                       + tMem + " \t" + fMemMax + " \t" + uMemMax + " \t" + (uMemMax - uMem));
  }
  )

}
