/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

//include(`Config_macros.m4')

package mqc;

__SECURITY__(
import mqc.security.Cipher;
import mqc.security.cipher.__SEC_CIPHER_ALG__;
)
__SECURITY_ORBKEY__(
import mqc.security.Key;
import mqc.security.RawSecretKey;
import mqc.security.InvalidKeyException;
)
__SECURITY_MD__(
import mqc.security.MessageDigest;
import mqc.security.md.__SEC_MD_ALG__;
)
__SECURITY_MAC__(
import mqc.security.Mac;
import mqc.security.mac.__SEC_MAC_ALG__;
)

/**
* Base ORB class
* 
* @author Olav Haugan
* @author David McKinnon
*/
public abstract class ORB
{
   __SECURITY__(
   public static Cipher cipherEncrypt;
   public static Cipher cipherDecrypt;
   )

   __SECURITY_MD__(
   public static int msgDigestLen;
   public static MessageDigest msgDigest;
   )
   __SECURITY_MAC__(
   public static int msgMacLen;
   public static Mac msgMac;
   )

   /**
   * Constructor.
   */
   public ORB()
   {
      __DEBUG__( System.out.println("ORB constructor");)
      __SECURITY__(
      __DEBUG__( System.out.println("Cipher is __SEC_CIPHER_ALG__");)
      cipherEncrypt = new __SEC_CIPHER_ALG__ ();
      cipherDecrypt = new __SEC_CIPHER_ALG__ ();
      )
      __SECURITY_MD__(
      msgDigest    = new __SEC_MD_ALG__ ();
      msgDigestLen = msgDigest.getDigestLength();
      )
      __SECURITY_MAC__(
      msgMac  = new __SEC_MAC_ALG__ (__SEC_MAC_ALG_PARAMS__);
      msgMacLen = msgMac.getMacLength();
      )
   }

   __SECURITY__(
   /**
   * Convert a user supplied 'key' into a usable session key
   *
   * @param  byte[] the cipher key value
   */
   public void initCipherKey(byte[] key) throws InvalidKeyException
   {
      byte[] keyMaterial = new byte[__SEC_CIPHER_KEY_LEN__];
      int klen = key.length;
      if (klen > __SEC_CIPHER_KEY_LEN__)
      {
         klen = __SEC_CIPHER_KEY_LEN__;
      }
      System.arraycopy(key, 0, keyMaterial, 0, klen);
      cipherEncrypt.init(Cipher.ENCRYPT_MODE, new RawSecretKey("__SEC_CIPHER_ALG__", keyMaterial));
      cipherDecrypt.init(Cipher.DECRYPT_MODE, new RawSecretKey("__SEC_CIPHER_ALG__", keyMaterial));
   }
   )

   __SECURITY_MAC__(
   /**
   * Convert a user supplied 'key' into a usable Mac key
   *
   * @param  byte[] the Mac key value
   */
   public void initMacKey(byte[] key) throws InvalidKeyException
   {
      byte[] keyMaterial = new byte[__SEC_MAC_KEY_LEN__];
      int klen = key.length;
      if (klen > __SEC_MAC_KEY_LEN__)
      {
         klen = __SEC_MAC_KEY_LEN__;
      }
      System.arraycopy(key, 0, keyMaterial, 0, klen);
      msgMac.init(new RawSecretKey("__SEC_MAC_ALG__", keyMaterial));
   }
   )

   __SECURITY_ORBKEY__(
   // use RawSecretKey instead of a custom ORBKey class
   )
}
